@@ -12,9 +12,8 @@ import sys
 
 # Local imports
 from spyder.utils import programs
-from spyder.config.base import _
+from spyder.config.base import _, is_pynsist
 from spyder.config.utils import is_anaconda
-from spyder.py3compat import PY2
 
 
 # =============================================================================
@@ -34,10 +33,13 @@ ATOMICWRITES_REQVER = '>=1.2.0'
 CHARDET_REQVER = '>=2.0.0'
 CLOUDPICKLE_REQVER = '>=0.5.0'
 DIFF_MATCH_PATCH_REQVER = '>=20181111'
-INTERVALTREE_REQVER = None
-IPYTHON_REQVER = ">=4.0;<6.0" if PY2 else ">=4.0"
-JEDI_REQVER = '>=0.17.1'
-KEYRING_REQVER = None
+# None for pynsist install for now
+# (check way to add dist.info/egg.info from packages without wheels available)
+INTERVALTREE_REQVER = None if is_pynsist() else '>=3.0.2'
+IPYTHON_REQVER = ">=7.6.0"
+JEDI_REQVER = '>=0.17.2'
+JSONSCHEMA_REQVER = '>=3.2.0'
+KEYRING_REQVER = '>=17.0.0'
 NBCONVERT_REQVER = '>=4.0'
 NUMPYDOC_REQVER = '>=0.6.0'
 PARAMIKO_REQVER = '>=2.4.0'
@@ -47,24 +49,31 @@ PICKLESHARE_REQVER = '>=0.4'
 PSUTIL_REQVER = '>=5.3'
 PYGMENTS_REQVER = '>=2.0'
 PYLINT_REQVER = '>=1.0'
-PYLS_REQVER = '>=0.34.0'
+PYLS_REQVER = '>=0.36.2'
+PYLS_BLACK_REQVER = '>=0.4.6'
+PYLS_SPYDER_REQVER = '>=0.3.0'
 PYXDG_REQVER = '>=0.26'
 PYZMQ_REQVER = '>=17'
 QDARKSTYLE_REQVER = '>=2.8'
 QTAWESOME_REQVER = '>=0.5.7'
-QTCONSOLE_REQVER = '>=4.6.0'
+QTCONSOLE_REQVER = '>=5.0.1'
 QTPY_REQVER = '>=1.5.0'
 RTREE_REQVER = '>=0.8.3'
+SETUPTOOLS_REQVER = '>=39.0.0'
 SPHINX_REQVER = '>=0.6.6'
-SPYDER_KERNELS_REQVER = '>=1.9.4;<1.10.0'
-WATCHDOG_REQVER = None
+SPYDER_KERNELS_REQVER = '>=1.10.1;<1.11.0'
+TEXTDISTANCE_REQVER = '>=4.2.0'
+THREE_MERGE_REQVER = '>=0.1.1'
+# None for pynsist install for now
+# (check way to add dist.info/egg.info from packages without wheels available)
+WATCHDOG_REQVER = None if is_pynsist() else '>=0.10.3'
 
 
 # Optional dependencies
 CYTHON_REQVER = '>=0.21'
 MATPLOTLIB_REQVER = '>=2.0.0'
 NUMPY_REQVER = '>=1.7'
-PANDAS_REQVER = '>=0.13.1'
+PANDAS_REQVER = '>=1.1.1'
 SCIPY_REQVER = '>=0.17.0'
 SYMPY_REQVER = '>=0.7.3'
 
@@ -110,12 +119,15 @@ DESCRIPTIONS = [
      'package_name': "jedi",
      'features': _("Main backend for the Python Language Server"),
      'required_version': JEDI_REQVER},
+    {'modname': 'jsonschema',
+     'package_name': 'jsonschema',
+     'features': _('Verify if snippets files are valid'),
+     'required_version': JSONSCHEMA_REQVER},
     {'modname': "keyring",
      'package_name': "keyring",
      'features': _("Save Github credentials to report internal "
                    "errors securely"),
-     'required_version': KEYRING_REQVER,
-     'display': sys.platform.startswith('linux') and not PY2},
+     'required_version': KEYRING_REQVER},
     {'modname': "nbconvert",
      'package_name': "nbconvert",
      'features': _("Manipulate Jupyter notebooks in the Editor"),
@@ -158,6 +170,15 @@ DESCRIPTIONS = [
      'package_name': 'python-language-server',
      'features': _("Code completion and linting for the Editor"),
      'required_version': PYLS_REQVER},
+    {'modname': 'pyls_black',
+     'package_name': 'pyls-black',
+     'features': _("Autoformat Python files in the Editor with the Black "
+                   "package"),
+     'required_version': PYLS_BLACK_REQVER},
+    {'modname': 'pyls_spyder',
+     'package_name': 'pyls-spyder',
+     'features': _('Spyder plugin for the Python Language Server'),
+     'required_version': PYLS_SPYDER_REQVER},
     {'modname': "xdg",
      'package_name': "pyxdg",
      'features': _("Parse desktop files on Linux"),
@@ -187,7 +208,11 @@ DESCRIPTIONS = [
      'package_name': "rtree",
      'features': _("Fast access to code snippets regions"),
      'required_version': RTREE_REQVER,
-     'display': is_anaconda()},
+     'display': is_anaconda() or is_pynsist()},
+    {'modname': "setuptools",
+     'package_name': "setuptools",
+     'features': _("Determine package version"),
+     'required_version': SETUPTOOLS_REQVER},
     {'modname': "sphinx",
      'package_name': "sphinx",
      'features': _("Show help for objects in the Editor and Consoles in a dedicated pane"),
@@ -196,6 +221,14 @@ DESCRIPTIONS = [
      'package_name': "spyder-kernels",
      'features': _("Jupyter kernels for the Spyder console"),
      'required_version': SPYDER_KERNELS_REQVER},
+    {'modname': 'textdistance',
+     'package_name': "textdistance",
+     'features': _('Compute distances between strings'),
+     'required_version': TEXTDISTANCE_REQVER},
+    {'modname': "three_merge",
+     'package_name': "three-merge",
+     'features': _("3-way merge algorithm to merge document changes"),
+     'required_version': THREE_MERGE_REQVER},
     {'modname': "watchdog",
      'package_name': "watchdog",
      'features': _("Watch file changes on project directories"),
@@ -271,10 +304,9 @@ class Dependency(object):
 
     def check(self):
         """Check if dependency is installed"""
-        if self.required_version and self.installed_version:
+        if self.required_version:
             return programs.is_module_installed(self.modname,
-                                                self.required_version,
-                                                self.installed_version)
+                                                self.required_version)
         else:
             return True
 
@@ -359,7 +391,7 @@ def missing_dependencies():
     """Return the status of missing dependencies (if any)"""
     missing_deps = []
     for dependency in DEPENDENCIES:
-        if not dependency.check() and dependency.kind != OPTIONAL:
+        if dependency.kind != OPTIONAL and not dependency.check():
             missing_deps.append(dependency)
 
     if missing_deps:
