@@ -54,6 +54,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.extra_selections_dict = {}
         self._restore_selection_pos = None
 
+        # Trailing newlines/spaces trimming
+        self.remove_trailing_spaces = False
+        self.remove_trailing_newlines = False
+
+        # Add a new line when saving
+        self.add_newline = False
+
         # Code snippets
         self.code_snippets = True
 
@@ -116,6 +123,15 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.tab_stop_width_spaces = tab_stop_width_spaces
         self.update_tab_stop_width_spaces()
 
+    def set_remove_trailing_spaces(self, flag):
+        self.remove_trailing_spaces = flag
+
+    def set_add_newline(self, add_newline):
+        self.add_newline = add_newline
+
+    def set_remove_trailing_newlines(self, flag):
+        self.remove_trailing_newlines = flag
+
     def update_tab_stop_width_spaces(self):
         self.setTabStopWidth(self.fontMetrics().width(
                              ' ' * self.tab_stop_width_spaces))
@@ -155,7 +171,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         Also assign draw orders to leave current_cell and current_line
         in the backgrund (and avoid them to cover other decorations)
 
-        NOTE: This will remove previous decorations added to  the same key.
+        NOTE: This will remove previous decorations added to the same key.
 
         Args:
             key (str) name of the extra selections group.
@@ -168,6 +184,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
         for selection in extra_selections:
             selection.draw_order = draw_order
+            selection.kind = key
 
         self.clear_extra_selections(key)
         self.extra_selections_dict[key] = extra_selections
@@ -191,7 +208,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             key (str) name of the extra selections group.
         """
         for decoration in self.extra_selections_dict.get(key, []):
-            self.decorations.remove(decoration, update=False)
+            self.decorations.remove(decoration)
         self.extra_selections_dict[key] = []
         self.update()
 
@@ -199,6 +216,31 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         """Emit changed signal"""
         self.modificationChanged.emit(self.document().isModified())
 
+    def get_visible_block_numbers(self):
+        """Get the first and last visible block numbers."""
+        first = self.firstVisibleBlock().blockNumber()
+        bottom_right = QPoint(self.viewport().width() - 1,
+                              self.viewport().height() - 1)
+        last = self.cursorForPosition(bottom_right).blockNumber()
+        return (first, last)
+
+    def get_buffer_block_numbers(self):
+        """
+        Get the first and last block numbers of a region that covers
+        the visible one plus a buffer of half that region above and
+        below to make more fluid certain operations.
+        """
+        first_visible, last_visible = self.get_visible_block_numbers()
+        buffer_height = round((last_visible - first_visible) / 2)
+
+        first = first_visible - buffer_height
+        first = 0 if first < 0 else first
+
+        last = last_visible + buffer_height
+        last = self.blockCount() if last > self.blockCount() else last
+
+        return (first, last)
+
     #------Highlight current line
     def highlight_current_line(self):
         """Highlight current line"""
@@ -217,11 +259,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     #------Highlight current cell
     def highlight_current_cell(self):
         """Highlight current cell"""
-        if not self.has_cell_separators or \
-          not self.highlight_current_cell_enabled:
+        if (not self.has_cell_separators or
+                not self.highlight_current_cell_enabled):
             return
-        cursor, whole_file_selected =\
-            self.select_current_cell()
+        cursor, whole_file_selected = self.select_current_cell()
         selection = TextDecoration(cursor)
         selection.format.setProperty(QTextFormat.FullWidthSelection,
                                      to_qvariant(True))
@@ -736,9 +777,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                                 QTextCursor.KeepAnchor)
         self.setTextCursor(cursor)
 
-    def delete_line(self):
+    def delete_line(self, cursor=None):
         """Delete current line."""
-        cursor = self.textCursor()
+        if cursor is None:
+            cursor = self.textCursor()
         if self.has_selected_text():
             self.extend_selection_to_complete_lines()
             start_pos, end_pos = cursor.selectionStart(), cursor.selectionEnd()
@@ -865,19 +907,21 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             # Handle completions for the internal console
             self.insert_text(text)
         else:
-            if self.code_snippets:
-                self.sig_insert_completion.emit(text)
-            else:
-                self.insert_text(text)
+            self.sig_insert_completion.emit(text)
             self.document_did_change()
 
     def is_completion_widget_visible(self):
         """Return True is completion list widget is visible"""
-        return self.completion_widget.isVisible()
+        try:
+            return self.completion_widget.isVisible()
+        except RuntimeError:
+            # This is to avoid a RuntimeError exception when the widget is
+            # already been deleted. See spyder-ide/spyder#13248.
+            return False
 
-    def hide_completion_widget(self):
+    def hide_completion_widget(self, focus_to_parent=True):
         """Hide completion widget and tooltip."""
-        self.completion_widget.hide()
+        self.completion_widget.hide(focus_to_parent=focus_to_parent)
         QToolTip.hideText()
 
     #------Standard keys
@@ -970,7 +1014,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         """Reimplemented to handle focus"""
         self.focus_changed.emit()
         self.focus_in.emit()
-        self.highlight_current_cell()
         QPlainTextEdit.focusInEvent(self, event)
 
     def focusOutEvent(self, event):
@@ -994,14 +1037,20 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                     elif event.delta() > 0:
                         self.zoom_in.emit()
                 return
+
         QPlainTextEdit.wheelEvent(self, event)
-        # Needed to prevent stealing focus to the find widget when scrolling
+
+        # Needed to prevent stealing focus when scrolling.
+        # If the current widget with focus is the CompletionWidget, it means
+        # it's being displayed in the editor, so we need to hide it and give
+        # focus back to the editor. If not, we need to leave the focus in
+        # the widget that currently has it.
         # See spyder-ide/spyder#11502
         current_widget = QApplication.focusWidget()
-        self.hide_completion_widget()
-        self.highlight_current_cell()
-        if current_widget is not None:
-            current_widget.setFocus()
+        if isinstance(current_widget, CompletionWidget):
+            self.hide_completion_widget(focus_to_parent=True)
+        else:
+            self.hide_completion_widget(focus_to_parent=False)
 
     def position_widget_at_cursor(self, widget):
         # Retrieve current screen height
