@@ -240,6 +240,7 @@ _qtaargs = {
     'DialogCloseButton':       [('fa.close',), {'color': MAIN_FG_COLOR}],
     'DirClosedIcon':           [('fa.folder-o',), {'color': MAIN_FG_COLOR}],
     'DialogHelpButton':        [('fa.life-ring',), {'color': 'darkred'}],
+    'VideoIcon':               [('fa5s.video',), {'color': MAIN_FG_COLOR}],
     'MessageBoxInformation':   [('fa.info',), {'color': MAIN_FG_COLOR}],
     'DirOpenIcon':             [('fa.folder-open',), {'color': MAIN_FG_COLOR}],
     'FileIcon':                [('fa.file-o',), {'color': MAIN_FG_COLOR}],
@@ -288,22 +289,39 @@ _qtaargs = {
     'undock':                  [('fa.external-link',), {'color': MAIN_FG_COLOR}],
     'dock':                    [('fa.caret-square-o-down',), {'color': MAIN_FG_COLOR}],
     'close_pane':              [('fa.window-close-o',), {'color': MAIN_FG_COLOR}],
-    # --- Autocompletion type icons --------------
+    # --- Autocompletion/document symbol type icons --------------
     'keyword':                 [('mdi.alpha-k-box',), {'color': '#df2935', 'scale_factor': BIG_ATTR_FACTOR}],
     'color':                   [('mdi.alpha-c-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'enum':                    [('mdi.alpha-e-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'value':                   [('mdi.alpha-v-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
+    'constant':                [('mdi.alpha-c-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'unit':                    [('mdi.alpha-u-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'text':                    [('mdi.alpha-t-box',), {'color': 'gray', 'scale_factor': BIG_ATTR_FACTOR}],
+    'file':                    [('mdi.alpha-f-box',), {'color': 'gray', 'scale_factor': BIG_ATTR_FACTOR}],
     'snippet':                 [('mdi.alpha-s-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'attribute':               [('mdi.alpha-a-box',), {'color': 'magenta', 'scale_factor': BIG_ATTR_FACTOR}],
     'reference':               [('mdi.alpha-r-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'variable':                [('mdi.alpha-v-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
-    'field':                   [('mdi.alpha-f-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
+    'field':                   [('mdi.alpha-a-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'property':                [('mdi.alpha-p-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
     'module':                  [('mdi.alpha-m-box',), {'color': '#daa520', 'scale_factor': BIG_ATTR_FACTOR}],
+    'namespace':               [('mdi.alpha-n-box',), {'color': '#daa520', 'scale_factor': BIG_ATTR_FACTOR}],
+    'package':                 [('mdi.alpha-p-box',), {'color': '#daa520', 'scale_factor': BIG_ATTR_FACTOR}],
     'class':                   [('mdi.alpha-c-box',), {'color':'#3775a9', 'scale_factor': BIG_ATTR_FACTOR}],
     'interface':               [('mdi.alpha-i-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
+    'string':                  [('mdi.alpha-s-box',), {'color': '#df2935', 'scale_factor': BIG_ATTR_FACTOR}],
+    'number':                  [('mdi.alpha-n-box',), {'color': '#df2935', 'scale_factor': BIG_ATTR_FACTOR}],
+    'boolean':                 [('mdi.alpha-b-box',), {'color': 'magenta', 'scale_factor': BIG_ATTR_FACTOR}],
+    'array':                   [('mdi.alpha-a-box',), {'color': '#df2935', 'scale_factor': BIG_ATTR_FACTOR}],
+    'object':                  [('mdi.alpha-o-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
+    'key':                     [('mdi.alpha-k-box',), {'color': 'yellow', 'scale_factor': BIG_ATTR_FACTOR}],
+    'class':                   [('mdi.alpha-c-box',), {'color':'#3775a9', 'scale_factor': BIG_ATTR_FACTOR}],
+    'null':                    [('mdi.alpha-n-box',), {'color': 'magenta', 'scale_factor': BIG_ATTR_FACTOR}],
+    'enum_member':             [('mdi.alpha-e-box',), {'color': '#e69c9c', 'scale_factor': BIG_ATTR_FACTOR}],
+    'struct':                  [('mdi.alpha-s-box',), {'color':'#3775a9', 'scale_factor': BIG_ATTR_FACTOR}],
+    'event':                   [('mdi.alpha-e-box',), {'color':'orange', 'scale_factor': BIG_ATTR_FACTOR}],
+    'operator':                [('mdi.alpha-o-box',), {'color':'orange', 'scale_factor': BIG_ATTR_FACTOR}],
+    'type_parameter':          [('mdi.alpha-t-box',), {'color':'orange', 'scale_factor': BIG_ATTR_FACTOR}],
     'private2':                [('spyder.circle-underscore',), {'color':'#e69c9c', 'scale_factor': SMALL_ATTR_FACTOR}],
     'private1':                [('spyder.circle-underscore',), {'color':'#e69c9c', 'scale_factor': SMALL_ATTR_FACTOR}],
     'method':                  [('mdi.alpha-m-box',), {'color':'#7ea67e', 'scale_factor': BIG_ATTR_FACTOR}],
